/* This program is used for testing several functions to produce random matrices.

It uses Eigen, a lightweight C++ template library for linear algebra.
Eigen itself is part of the KDE project.  Copyright (C) 2008 Gael Guennebaud <g.gael@free.fr>
Eigen is used here under the LGPL license.  Eigen files are in the Eigen folder.

*/

/* TO COMPILE THIS PROGRAM with gcc, type [ g++ -I eigenfiles RandomMatrix.cpp -o RandomMatrix ]. */

#include<iostream>
#include<iomanip>
#include<fstream>
#include<cmath>
#include<cstdlib>
#include<string>
#include<vector>
#include<cstdio>
#include<ctime>
#include<cassert>

#include<Eigen/Core>		// this is needed to use Eigen
#include<Eigen/QR>

using namespace std;
USING_PART_OF_NAMESPACE_EIGEN


/* This function overwrites a matrix with (real) elements uniformly chosen from a certain interval.
INPUT: Eigen-class dynamic-size matrix of doubles, double min and max random values
OUTPUT: void (overwrites input matrix)
*/
void randomize_everything(MatrixXd& inmat, const double min, const double max){

  srand(time(0));						// initialize random number generator

  for(int i=0; i < inmat.rows(); i++){
    for(int j=0; j < inmat.cols(); j++){
	  inmat(i,j) = min + (max - min) * (double)rand() / RAND_MAX;
    }
  }
}


/* This function overwrites a matrix to make it real and diagonal but otherwise random.
INPUT: Eigen-class dynamic-size matrix of doubles, double min and max random values
OUTPUT: void (overwrites input matrix)
*/
void randomize_diagonal(MatrixXd& inmat, const double min, const double max){

  srand(time(0));						// initialize random number generator

  for(int i=0; i < inmat.rows(); i++){
    for(int j=0; j < inmat.cols(); j++){
	  if(i==j){ inmat(i,j) = min + (max - min) * (double)rand() / RAND_MAX; }
	  else{ inmat(i,j) = 0.0; }			// off-diagonal elements will all be zero
    }
  }
}


/* This function overwrites a square matrix to make it real and symmetric but otherwise random.  First
it fills diagonals with nonzero numbers, then the rest of the right-triangle with random doubles between a and b.
Then the left-triangle is filled in as needed to make the matrix symmetric.
INPUT: dynamic SQUARE double matrix (by reference) from Eigen's MatrixXd class
OUTPUT: void (overwrites input matrix)
*/
void randomize_symmetric(MatrixXd& inmat, const double min, const double max){

  srand(time(0));						// initialize random number generator
  int rowsize = inmat.rows();
  int colsize = inmat.cols();
  assert( rowsize == colsize );			// check that matrix is actually square, else kill program
  
  for(int i=0; i < rowsize; i++){
	for(int j=i; j < colsize; j++){		// fill in the right-triangular part with rands
	  inmat(i,j) = min + (max - min) * (double)rand() / RAND_MAX;
    }
  }
  
  for(int i=0; i < rowsize; i++){			// fill in the rest to make the matrix symmetric
    for(int j=0; j < i; j++){
	  inmat(i,j) = inmat(j,i);
    }
  }
}


/* This function prints a matrix to screen in an easy-to-read way. (Avoid printing a 1000x1000 matrix!)
INPUT: any Eigen-class matrix, desired precision, int desired column width.
OUTPUT: void (prints to cout).
*/
void printmatrix(MatrixXd& matrix, const int precision, const int width){

  cout << setprecision(precision) << endl;
  for(int i=0; i < matrix.rows(); i++){
	for(int j=0; j < matrix.cols(); j++){
      cout << setw(width) << matrix(i,j);
    }
	cout << endl;
  }
  cout << endl;
}

int main(){

  int dimension;
  cout << "What size matrix? ";
  cin >> dimension;
  int min = 0;
  int max = 1;
  cout << endl << "What range of random values? ";
  cin >> min >> max;
  
  cout << endl << "Initializing random matrices..." << endl;
  
  // random symmetric matrix
  MatrixXd Amatrix(dimension, dimension);
  randomize_symmetric(Amatrix, min, max);
  cout << "Here is a random symmetric matrix:" << endl;
  printmatrix(Amatrix, 5, 10);
  
  Eigen::SelfAdjointEigenSolver<MatrixXd> Asolved(Amatrix);
  VectorXd A_evalues = Asolved.eigenvalues();
  cout << "Its eigenvalues are:" << setprecision(5);
  for(int i=0; i < dimension; i++){
    cout << setw(10) << A_evalues(i);
  }
  cout << endl;
  
  
  // random diagonal matrix
  MatrixXd Bmatrix(dimension, dimension);
  randomize_diagonal(Bmatrix, min, max);
  cout << "Here is a random diagonal matrix:" << endl;
  printmatrix(Bmatrix, 5, 10);


  // random positive-definite symmetric matrix made by squaring a random symmetric matrix
  MatrixXd Cmatrix(dimension, dimension);
  randomize_symmetric(Cmatrix, min, max);
  MatrixXd SquaredCmatrix = Cmatrix * Cmatrix;
  cout << "Here is a random positive-definite symmetric matrix:" << endl;
  printmatrix(SquaredCmatrix, 5, 10);

  Eigen::SelfAdjointEigenSolver<MatrixXd> SquaredCsolved(SquaredCmatrix);
  VectorXd C_evalues = SquaredCsolved.eigenvalues();
  cout << "Its eigenvalues are:" << setprecision(5);
  for(int i=0; i < dimension; i++){
    cout << setw(10) << C_evalues(i);
  }
  cout << endl;
  
	  
  // totally random matrix
  MatrixXd Dmatrix(dimension, dimension);
  randomize_everything(Dmatrix, min, max);
  cout << "Here is a completely random matrix:" << endl;
  printmatrix(Dmatrix, 5, 10);
  

  /*
  // Open the target file for the results

  ofstream results;
  results.open ("TESTFILE.txt");

  results << "The number you typed is: " << x << endl;

  // Don't forget to close the file!

  results.close(); 
  */
  
return(0);

}
